from pathlib import Path

from peek_plugin_base.server.PeekStoragePlatformHookABC import (
    PeekStoragePlatformHookABC,
)

from peek_platform import PeekPlatformConfig
from peek_plugin_base.server.PeekPlatformAdminHttpHookABC import (
    PeekPlatformAdminHttpHookABC,
)
from peek_plugin_base.server.PeekPlatformServerHttpHookABC import (
    PeekPlatformServerHttpHookABC,
)


class PeekStoragePlatformHook(PeekStoragePlatformHookABC):
    def __init__(self, pluginName: str) -> None:
        PeekPlatformAdminHttpHookABC.__init__(self)
        PeekPlatformServerHttpHookABC.__init__(self)
        self._pluginName = pluginName

    @property
    def dbConnectString(self) -> str:
        from peek_platform import PeekPlatformConfig

        return PeekPlatformConfig.config.dbConnectString

    @property
    def fileStorageDirectory(self) -> Path:
        from peek_platform import PeekPlatformConfig

        return Path(PeekPlatformConfig.config.pluginDataPath(self._pluginName))

    def _getOtherPluginApi(self, pluginName):
        serverPluginLoader = PeekPlatformConfig.pluginLoader

        otherPlugin = serverPluginLoader.pluginEntryHook(pluginName)
        if not otherPlugin:
            return None

        from peek_plugin_base.server.PluginLogicEntryHookABC import (
            PluginLogicEntryHookABC,
        )

        assert isinstance(
            otherPlugin, PluginLogicEntryHookABC
        ), "Not an instance of PluginLogicEntryHookABC"

        return otherPlugin

    def getOtherPluginApi(self, pluginName):
        otherPlugin = self._getOtherPluginApi(pluginName)
        return otherPlugin.publishedServerApi if otherPlugin else None

    @property
    def serviceId(self) -> str:
        import socket

        return "server|" + socket.gethostname()



import sys
from collections import namedtuple
from typing import Callable, Dict, Optional

import json
from sqlalchemy import func
from sqlalchemy.dialects import postgresql
from sqlalchemy.sql import Select

from peek_plugin_base.storage.DbConnection import DbSessionCreator

_LoadPayloadTupleResult = namedtuple(
    "_LoadPayloadTupleResult", ["count", "encodedPayload"]
)

__sysPathsJson = json.dumps(sys.path)


def callPGLoadPayloadTuplesBlocking(
    dbSessionCreator: DbSessionCreator,
    sql: Select,
    sqlCoreLoadTupleClassmethod: Callable,
    payloadFilt: Optional[Dict] = None,
    fetchSize=50,
) -> _LoadPayloadTupleResult:
    payloadFileJson = json.dumps(payloadFilt if payloadFilt else {})

    sqlStr = str(
        sql.compile(
            dialect=postgresql.dialect(), compile_kwargs={"literal_binds": True}
        )
    )

    loaderModuleClassMethodStr = ".".join(
        [
            sqlCoreLoadTupleClassmethod.__self__.__module__,
            sqlCoreLoadTupleClassmethod.__self__.__name__,
            sqlCoreLoadTupleClassmethod.__name__,
        ]
    )

    session = dbSessionCreator()
    try:
        sqlFunc = func.peek_storage_service.load_paylaod_tuples(
            sqlStr,
            payloadFileJson,
            loaderModuleClassMethodStr,
            __sysPathsJson,
            fetchSize,
        )

        resultJsonStr: str = next(session.execute(sqlFunc))[0]

        resultJson: Dict = json.loads(resultJsonStr)
        if resultJson["encodedPayload"]:
            resultJson["encodedPayload"] = resultJson["encodedPayload"].encode()

        return _LoadPayloadTupleResult(**resultJson)

    finally:
        session.close()



import json
import sys
from typing import Callable, Any

import json
from sqlalchemy import func
from vortex.Tuple import Tuple, addTupleType

from peek_plugin_base.storage.DbConnection import DbSessionCreator

__sysPathsJson = json.dumps(sys.path)


@addTupleType
class _RunPyInWorkerArgsTuple(Tuple):
    __tupleType__ = "peek_storage_service." + "_RunPyInWorkerArgsTuple"
    __slots__ = ("args", "kwargs")


@addTupleType
class _RunPyInWorkerResultTuple(Tuple):
    __tupleType__ = "peek_storage_service." + "_RunPyInWorkerResultTuple"
    __slots__ = ("result",)


def runPyWorkerTaskInPgBlocking(
    dbSessionCreator: DbSessionCreator,
    sqlaUrl: str,
    moduleMethodToRun: Callable,
    *argsTuple,
    **kwargs
) -> Any:
    argsTuple = _RunPyInWorkerArgsTuple(args=argsTuple, kwargs=kwargs)

    funcPath = moduleMethodToRun.__repr__().strip("<>").split(" ")[1]

    session = dbSessionCreator()
    try:
        sqlFunc = func.peek_storage_service.run_worker_task_python(
            sqlaUrl, json.dumps(argsTuple.toJsonDict()), funcPath, __sysPathsJson
        )

        resultJsonStr: str = next(session.execute(sqlFunc))[0]
        session.commit()

        resultTuple = _RunPyInWorkerResultTuple().fromJsonDict(
            json.loads(resultJsonStr)
        )

        return resultTuple.result

    finally:
        session.close()
